/*
 * astrosrv.c
 * Copyright (C) Damian Walker 2000
 *
 * Astroserv - Main Program
 *
 * $Id: astrosrv.c,v 1.3 2001/02/15 16:38:19 damian Exp $
 *
 * $Log: astrosrv.c,v $
 * Revision 1.3  2001/02/15 16:38:19  damian
 * Cleaned up logging and screen output
 *
 * Revision 1.2  2000/07/27 09:31:18  damian
 * Cosmetic changes to heading comments
 *
 * Revision 1.1.1.1  2000/07/27 09:18:06  damian
 * Initial release of Astroserv
 *
 * Revision 1.1  2000/05/31 10:17:51  damian
 * Initial revision
 *
 */


/* included headers ***************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "astrosrv.h"
#include "turnin.h"
#include "trnout.h"
#include "eduser.h"
#include "edplyr.h"
#include "scrlog.h"


/* enumerated types ***************************************************/


typedef enum {
    S_NONE,    /* no service selected */
    S_TURNIN,  /* turn input selected */
    S_TURNOUT, /* turn output selected */
    S_EDUSER,  /* user editor selected */
    S_EDPLYR,  /* player editor selected */
    S_HELP,    /* program help selected */
    S_ERROR    /* error on command line */
} service;


/* global variables ***************************************************/


char pid[36] = PROGNAME;


/* level 1 routines ***************************************************/


/* astrosrv_args() - parse command line */
service astrosrv_args(char *path, char **argp, int argc, char **argv)
{
    int     argi, /* current input parameter (argv) */
            argo; /* current output parameter (argp) */
    service s;    /* service to return */
    char   *env;  /* pointer to environment variable */

    /* initialise */
    strcpy(path, "");
    if(( env = getenv("ASTROWAR") ) != NULL) strcpy(path, env);
    argi = 1; argo = 0; s = S_NONE;

    /* loop through parameters */
    while(argi < argc)
    {
        if( !strcasecmp(argv[argi], "--in") && s == S_NONE )
            s = S_TURNIN;
        else if( !strcasecmp(argv[argi], "--out") && s == S_NONE )
            s = S_TURNOUT;
        else if( !strcasecmp(argv[argi], "--user") && s == S_NONE )
            s = S_EDUSER;
        else if( !strcasecmp(argv[argi], "--player") && s == S_NONE )
            s = S_EDPLYR;
        else if( !strcasecmp(argv[argi], "--help") )
            s = S_HELP;
        else if( !strcasecmp(argv[argi], "--path") )
        {
            if(++argi < argc)
                strcpy(path, argv[argi]);
            else
                s = S_ERROR;
        }
        else if(argo < 10)
            argp[argo++] = argv[argi];
        else s = S_ERROR;
        argi++;
    }

    /* if no command chosen, assume 'in' */
    if(s == S_NONE) s = S_TURNIN;

    /* fill rest of output parameters and return */
    while(argo < 10) argp[argo++] = NULL;
    return s;
}

/* astrosrv_drop() - drop semaphore */
result astrosrv_drop(char *path)
{
    char  filename[128]; /* full name of file */
    FILE *sem;           /* file handle for semaphore */

    sprintf(filename, "%sastrowar.sem", path);
    if(( sem = fopen(filename, "r") ) != NULL)
    {
        fclose(sem);
        return R_EXISTS;
    }
    if(( sem = fopen(filename, "w") ) == NULL) return R_FILE;
    fputs("astroserv\n", sem);
    fclose(sem);

    return R_OK;
}

/* astrosrv_lift() - lift a semaphore */
void astrosrv_lift(char *path)
{
    char filename[128]; /* full name of file */

    sprintf(filename, "%sastrowar.sem", path);
    remove(filename);
}


/* level 0 routines ***************************************************/


/* main program */
int main(int argc, char **argv)
{
    service s;        /* service requested on command line */
    char   *argp[10], /* process-specific parameters */
            path[72]; /* path to configs, semaphores etc. */
    result  r;        /* returned to calling process */

    scrlog_open();
    s = astrosrv_args(path, argp, argc, argv);
    if(( r = astrosrv_drop(path) ) == R_OK)
    {
        switch(s)
        {
            case S_TURNIN:  r = turnin_main(path, argp);
                            break;
            case S_TURNOUT: r = trnout_main(path, argp);
                            break;
            case S_EDUSER:  r = eduser_main(path, argp);
                            break;
            case S_EDPLYR:  r = edplyr_main(path, argp);
                            break;
            case S_HELP:    r = R_OK;
                            scrlog_help();
                            break;
            default:        scrlog_error();
        }
        astrosrv_lift(path);
    }
    scrlog_close();

    return r;
}
