/*
 * config.h
 * Copyright (C) Damian Walker 2000
 *
 * Astroserv - Configuration File Maintenance
 *
 * $Id: config.c,v 1.3 2001/02/15 16:38:21 damian Exp $
 *
 * $Log: config.c,v $
 * Revision 1.3  2001/02/15 16:38:21  damian
 * Cleaned up logging and screen output
 *
 * Revision 1.2  2000/07/27 09:31:22  damian
 * Cosmetic changes to heading comments
 *
 * Revision 1.1.1.1  2000/07/27 09:18:08  damian
 * Initial release of Astroserv
 *
 * Revision 1.1  2000/05/31 10:17:51  damian
 * Initial revision
 *
 */

/*
 * Included Headers
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "astrosrv.h"
#include "log.h"

/* 
 * Structures 
 */

/* configuration record */
typedef struct {
    char folder[72], /* netmail directory */
      logfile[72], /* name of log file */
      mailprog[72], /* program to send mail */
      lockfile[72]; /* mailbox lock file */
    int loglevel; /* logging level */
} config;

/*
 * Global Variables
 */

extern char pid[36]; /* program ID */
static config config_rec;   /* configuration record */

/*
 * Level 1 Routines
 */

/* config_intfolder() - interpret 'folder' config item */
void config_intfolder(void)
{
    char *tok; /* log filename token */

    tok = strtok(NULL, "\n");
    if( strlen(tok) > 71 )
    {
        strncpy(config_rec.folder, tok, 71);
        config_rec.folder[71] = '\0';
    }
    else strcpy(config_rec.folder, tok);
}

/* config_intlogfile() - interpret 'log file' config item */
void config_intlogfile(void)
{
    char *tok; /* log filename token */

    tok = strtok(NULL, "\n");
    if( strlen(tok) > 71 )
    {
        strncpy(config_rec.logfile, tok, 71);
        config_rec.logfile[71] = '\0';
    }
    else strcpy(config_rec.logfile, tok);
}

/* config_intmailprog() - interpret 'mailprog' config item */
void config_intmailprog(void)
{
    char *tok; /* log filename token */

    tok = strtok(NULL, "\n");
    if( strlen(tok) > 71 )
    {
        strncpy(config_rec.mailprog, tok, 71);
        config_rec.mailprog[71] = '\0';
    }
    else strcpy(config_rec.mailprog, tok);
}

/* config_intlockfile() - interpret 'lockfile' config item */
void config_intlockfile(void)
{
    char *tok; /* log filename token */

    tok = strtok(NULL, "\n");
    if( strlen(tok) > 71 )
    {
        strncpy(config_rec.lockfile, tok, 71);
        config_rec.lockfile[71] = '\0';
    }
    else strcpy(config_rec.lockfile, tok);
}

/* interpret logging level config item */
void config_intloglevel(void) {

  /* local variables */
  char *tok; /* log level token */

  /* get and interpret token */
  tok = strtok (NULL, "\n");
  config_rec.loglevel = 0;
  if (tok) {
    if (strchr (tok, '!')) config_rec.loglevel |= LOGERROR;
    if (strchr (tok, '?')) config_rec.loglevel |= LOGWARNING;
    if (strchr (tok, '*')) config_rec.loglevel |= LOGMAJOR;
    if (strchr (tok, '+')) config_rec.loglevel |= LOGMINOR;
    if (strchr (tok, '-')) config_rec.loglevel |= LOGTRIVIAL;
    if (strchr (tok, '#')) config_rec.loglevel |= LOGDEBUG;
  }

}

/*
 * Level 0 Routines 
 */

/* config_open() - intialise the library */
result config_open(char *path)
{
    strcpy(config_rec.folder, "");
    sprintf(config_rec.logfile, "%sastrosrv.log", path);
    strcpy(config_rec.mailprog, "/bin/mail");
    return R_OK;
}

/* config_close() - clean up and close */
void config_close(void)
{
}

/* config_read() - open (and read) the configuration file */
result config_read(char *path, int *line)
{
    char      filename[128]; /* full filename of config file */
    FILE     *config_file;   /* config file handle */
    char      buf[1024],     /* buffer for line input */
             *tok;           /* pointer to token in buf */

    /* attempt to open configuration file */
    sprintf(filename, "%sastrosrv.data", path);
    if(( config_file = fopen(filename, "r") ) == NULL )
        return R_FILE;
    config_rec.loglevel = LOGERROR | LOGWARNING | LOGMAJOR | LOGMINOR |
      LOGTRIVIAL;

    /* read and interpret config lines */
    *line = 0;
    while( fgets(buf, 1023, config_file) )
    {
        ++*line;
        if(*buf != '\n' && *buf != ';' && *buf != '\0')
        {
            if( strchr(buf, '\n') == NULL ) strcat(buf, "\n");
            tok = strtok(buf, " \n");
            if( !strcasecmp(tok, "folder") )
                config_intfolder();
            else if( !strcasecmp(tok, "logfile") )
                config_intlogfile();
            else if( !strcasecmp(tok, "mailprog") )
                config_intmailprog();
	    else if (!strcasecmp(tok, "lockfile"))
	        config_intlockfile();
	    else if (!strcasecmp (tok, "loglevel"))
		config_intloglevel ();
            else
            {
                fclose(config_file);
                return R_BADCMD;
            }
        }
    }

    /* close and clean up */
    fclose(config_file);
    return R_OK;
}

/* config_write() - write the config to a file */
result config_write(char *path)
{
    char      filename[128]; /* full filename of config file */
    FILE     *config_file;   /* config file handle */

    /* attempt to open configuration file */
    sprintf(filename, "%sastrosrv.data", path);
    if(( config_file = fopen(filename, "w") ) == NULL )
        return R_FILE;

    /* output title */
    fprintf(config_file,
        ";   Astroserv Configuration File %sastrosrv.data\n"
        ";   automatically generated by %s\n", path, pid);

    /* output folder */
    fprintf(config_file,
        "\n; folder <file>\n"
          ";   set the inbound mailbox file.\n");
    fprintf(config_file, "\nfolder %s\n", config_rec.folder);

    /* output logfile name */
    fprintf(config_file,
        "\n; logfile <logname>\n"
          ";   sets the log output file to <logname>.  ");
    fprintf(config_file, "\nlogfile %s\n", config_rec.logfile);

    /* output mailprog */
    fprintf(config_file,
        "\n; mailprog <file> [<switches>]\n"
          ";   sets the program used to post mail");
    fprintf(config_file, "\nmailprog %s\n", config_rec.mailprog);

    /* output lockfile */
    if (*config_rec.lockfile) {
      fprintf(config_file,
	      "\n; lockfile <file>\n"
	      ";   mailbox lockfile file");
      fprintf(config_file, "\nlockfile %s\n", config_rec.lockfile);
    }

    /* output log level */
    if (config_rec.loglevel != (LOGERROR | LOGWARNING | LOGMAJOR | LOGMINOR
	| LOGTRIVIAL) ) {
	fprintf (config_file,
		 "\n; loglevel <characters>\n"
		 ";   levels of log information (!, ?, *, +, -, #");
	fprintf (config_file, "\nloglevel ");
	if (config_rec.loglevel & LOGERROR) fputc ('!', config_file);
	if (config_rec.loglevel & LOGWARNING) fputc ('?', config_file);
	if (config_rec.loglevel & LOGMAJOR) fputc ('*', config_file);
	if (config_rec.loglevel & LOGMINOR) fputc ('+', config_file);
	if (config_rec.loglevel & LOGTRIVIAL) fputc ('-', config_file);
	if (config_rec.loglevel & LOGDEBUG) fputc ('#', config_file);
	fputc ('\n', config_file);
    }

    /* close file */
    fclose(config_file);
    return R_OK;
}

/* config_set...() - set field values */
char *config_setfolder(char *folder)
    { return strcpy(config_rec.folder, folder); }
char *config_setlogfile(char *logfile)
    { return strcpy(config_rec.logfile, logfile); }
char *config_setmailprog(char *mailprog)
    { return strcpy(config_rec.mailprog, mailprog); }
char *config_setlockfile(char *lockfile)
    { return strcpy(config_rec.lockfile, lockfile); }
int config_setloglevel (int loglevel)
    { return config_rec.loglevel = loglevel; }

/* config_get...() - get field values */
char *config_getfolder(char *folder)
    { return strcpy(folder, config_rec.folder); }
char *config_getlogfile(char *logfile)
    { return strcpy(logfile, config_rec.logfile); }
char *config_getmailprog(char *mailprog)
    { return strcpy(mailprog, config_rec.mailprog); }
char *config_getlockfile(char *lockfile)
    { return strcpy(lockfile, config_rec.lockfile); }
int config_getloglevel (void)
    { return config_rec.loglevel; }
