/*
 * eduser.c
 * Copyright (C) Damian Walker 2000
 *
 * Astroserv - User Database Edit Module
 *
 * $Id: eduser.c,v 1.4 2001/02/15 16:38:23 damian Exp $
 *
 * $Log: eduser.c,v $
 * Revision 1.4  2001/02/15 16:38:23  damian
 * Cleaned up logging and screen output
 *
 * Revision 1.3  2000/08/24 13:22:47  damian
 * User number wasn't being returned properly.  This is a quick and ugly hack
 * to get around this.
 *
 * Revision 1.2  2000/07/27 09:31:25  damian
 * Cosmetic changes to heading comments
 *
 * Revision 1.1.1.1  2000/07/27 09:18:08  damian
 * Initial release of Astroserv
 *
 * Revision 1.1  2000/05/31 10:17:51  damian
 * Initial revision
 *
 */


/* included headers ***************************************************/


#include <stdio.h>
#include <string.h>
#include "astrosrv.h"
#include "config.h"
#include "user.h"
#include "scrlog.h"


/* enumerated types ***************************************************/


/* eduser services */
typedef enum {
  S_NONE,   /* no service requested */
  S_ADD,    /* add a new user record */
  S_EDIT,   /* edit a user record */
  S_DELETE, /* delete a user record */
  S_LIST,   /* list all or selected user records */
  S_ERROR   /* error in command line */
} service;


/* global variable ****************************************************/


/* due to a bug in C, the following needs to be global. */
static long unum;     /* user number */


/* level 1 routines ***************************************************/


/* eduser_open() - open user editor */
result eduser_open(char *path)
{
  result
    r;                /* returned to calling process */
  char
    logfilename[128]; /* name and path of log file */
  int
    line;             /* line on which config read failed */

  /* attempt to read configuration */
  if(( r = config_open(path) ))
    {
      scrlog_badreturncode(r, "config_open");
      return r;
    }
  switch( r = config_read(path, &line) )
    {
    case R_FILE:
      scrlog_cannotreadconfig(path);
      return r;
    case R_BADCMD:
      scrlog_badcommandinconfig(path, line);
      return r;
    case R_OK:
      break;
    default:
      scrlog_badreturncode(r, "config_read");
      return r;
    }

  /* attempt to open log file */
  config_getlogfile(logfilename);
  if(( r = scrlog_openlog(logfilename, config_getloglevel ()) ))
    {
      scrlog_cannotopenlog(logfilename);
      goto closeconfig;
    }
  scrlog_eduser();

  /* open user file */
  switch( r = user_open(path) )
    {
    case R_FILE:
      if(!( r = user_create(path) ))
	{
	  scrlog_filecreated(path, "suser.data");
	  if(!( r = user_open(path) ))
	    {
	      scrlog_fileopened(path, "suser.data");
	      break;
	    }
	}
      scrlog_cannotopenfile(path, "suser.data");
      goto closeconfig;
    case R_OK:
      scrlog_fileopened(path, "suser.data");
      break;
    case R_HEADER:
      scrlog_badheaderinfile(path, "suser.data");
      goto closeconfig;
    default:
      scrlog_badreturncode(r, "user_open");
      goto closeconfig;
    }

  /* all OK, return */
  return R_OK;

  /* deal with errors */
closeconfig:
  config_close();
  return r;
}

/* eduser_interpret() - interpret command line arguments */
service eduser_interpret(char *addr, char **argp)
{
  int
    argc,  /* argument count */
    argn;  /* number of arguments */
  service
    s;     /* service to return */
  long
    num;   /* number read from command line */
  enum {   /* state variable */
    T_NONE,  /* not expecting anything in particular */
    T_ADDR   /* expecting an address */
  } t;

  /* initialise variables */
  s = S_NONE;
  unum = 0;
  strcpy(addr, "");
  t = T_NONE;

  /* find last argument */
  argn = 10;
  for(argc = 9; argc >= 0; --argc) if( !argp[argc] ) argn = argc;

  /* interpret arguments */
  for(argc = 0; argc < argn && s != S_ERROR; ++argc)
    {
/**/  printf ("DEBUG: Processing arg %s\n", argp[argc]);
      if( !strcasecmp( argp[argc], "--add" ) && s == S_NONE )
	s = S_ADD;
      else if( !strcasecmp( argp[argc], "--edit" ) && s == S_NONE )
	s = S_EDIT;
      else if( !strcasecmp( argp[argc], "--delete" ) && s == S_NONE )
	s = S_DELETE;
      else if( !strcasecmp( argp[argc], "--list" ) && s == S_NONE )
	s = S_LIST;
      else if( sscanf( argp[argc], "%ld", &num ) == 1 && !unum ) {
	unum = num;
/**/	printf ("DEBUG: Editing user %ld\n", unum);
      } else if( !strcasecmp( argp[argc], "--addr" ) && t == T_NONE )
	t = T_ADDR;
      else if( t == T_ADDR )
	{ strncpy( addr, argp[argc], 72 ); t = T_NONE; }
      else
	s = S_ERROR;
    }
  if(t) s = S_ERROR;

  return s;
}

/* eduser_add() - add a user record to the user database */
void eduser_add(char *addr)
{
  user
    *u = user_new(); /* new user record */
  long
    unum;            /* user number for new record */

  /* ascertain next available user number */
  if( user_last(u, U_NUMBER) )
    unum = 1;
  else
    unum = user_getnumber(u) + 1;
  scrlog_useradd(unum);

  /* validate user details */
  if(!*addr)
    {
      scrlog_baduserdetails(unum);
      user_old(u);
      return;
    }

  /* create user details */
  user_clear(u);
  user_setnumber(u, unum);
  user_setaddr(u, addr);

  /* save and clean up */
  user_write(u);
  user_old(u);
}

/* eduser_edit() - edit a user record in the user database */
void eduser_edit(long unum, char *addr)
{
  user
    *u = user_new(); /* user record to edit */
  char
    searchtext[11];  /* user number to search for */

  /* log message */
  scrlog_useredit(unum);

  /* find user record in file */
  sprintf(searchtext, "%ld", unum);
  if( user_find(u, searchtext) )
    {
      user_old(u);
      scrlog_baduserdetails(unum);
      return;
    }

  /* change settings which were specified */
  if(*addr)
    user_setaddr(u, addr);

  /* save record and clean up */
  user_write(u);
  user_old(u);
}

/* eduser_delete() - delete a user from the user database */
void eduser_delete(long unum, char *addr)
{
  user *u = user_new(); /* user to delete */
  int deleteyes;        /* !0 if the user should be deleted */
  char uaddr[72];       /* user address in user file */
  char
    searchtext[11];  /* user number to search for */

  /* log message */
  scrlog_userdelete(unum);

  /* find user record */
  sprintf(searchtext, "%ld", unum);
  if( user_find(u, searchtext) )
    {
      scrlog_baduserdetails(unum);
      user_old(u);
      return;
    }

  /* verify user details (supplied as safety catch) */
  deleteyes = !0;
  user_getaddr(uaddr, u);
  if( strcasecmp(addr, uaddr) )
    deleteyes = 0;

  /* delete record and clean up */
  if(deleteyes)
    user_delete(u);
  else
    scrlog_baduserdetails(unum);
  user_old(u);
}

/* eduser_list() - list users in the user database */
void eduser_list(long unum, char *addr)
{
  user *u = user_new(); /* user record */
  result r;             /* result of file read */
  long uunum;           /* user number in user file */
  char uaddr[72];       /* user address in user file */
  int listyes;          /* !0 = list this record */

  scrlog_listheader();
  r = user_first(u, U_NUMBER);
  while(!r)
    {
      /* get details */
      uunum = user_getnumber(u);
      user_getaddr(uaddr, u);

      /* check against selection criteria */
      listyes = 1;
      if(unum && uunum != unum)
	listyes = 0;
      else if(*addr && strcasecmp(uaddr, addr) )
	listyes = 0;

      /* write record if OK & move on */
      if(listyes)
        scrlog_listbody(uunum, uaddr);
      r = user_next(u, U_NUMBER);
    }
  scrlog_listfooter();

  /* clean up and return */
  user_old(u);
}

/* eduser_close() - close files in use */
void eduser_close(void)
{
  user_close();
  config_close();
  scrlog_finished();
}


/* level 0 routines ***************************************************/


/* eduser_main() - main user editor function */
result eduser_main(char *path, char **argp)
{
  result r;      /* returned to calling process */
  char addr[36]; /* user name */
  service s;     /* service requested */

  if(( r = eduser_open(path) )) return r;
  switch( s = eduser_interpret(addr, argp) )
    {
    case S_NONE: case S_ERROR:
      scrlog_error();
      return R_BADCMD;
    case S_ADD:
      eduser_add(addr);
      break;
    case S_EDIT:
/**/  printf ("DEBUG: Editing user %ld\n", unum);
      eduser_edit(unum, addr);
      break;
    case S_DELETE:
      eduser_delete(unum, addr);
      break;
    case S_LIST:
      eduser_list(unum, addr);
      break;
    }
  eduser_close();
  if(s == S_DELETE) user_pack(path);

  return R_OK;
}
