/*
 * iomsg.c
 * Copyright (C) Damian Walker 2000
 *
 * Astroserv - Input Message Processing
 *
 * $Id: iomsg.c,v 1.2 2000/07/27 09:31:27 damian Exp $
 *
 * $Log: iomsg.c,v $
 * Revision 1.2  2000/07/27 09:31:27  damian
 * Cosmetic changes to heading comments
 *
 * Revision 1.1.1.1  2000/07/27 09:18:09  damian
 * Initial release of Astroserv
 *
 * Revision 1.1  2000/05/31 10:17:51  damian
 * Initial revision
 *
 */

/*
 * Included Headers
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "astrosrv.h"
#include "config.h"

/*
 * Global Variables
 */

static FILE *folder_file; /* handle for mail folder */
char game_name[16]; /* game that the user wants to play */
char from_address[72]; /* the address of the user */

/*
 * Level 1 Routines
 */

/* interpret an email address */
static void interpret_email_address (char *out_address, char *in_address) {

  /* local variables */
  char address_buffer[128]; /* buffer for working on address */
  char *ptr; /* pointer for working on address */
  char dump[128]; /* buffer for working on parts of address line */

  /* search for an email address */
  strcpy (address_buffer, in_address);
  ptr = strtok (address_buffer, " \n");
  while (ptr) {

    /* is this an address? */
    if (sscanf (ptr, "%[^@]@%s", dump, dump) == 2)
      strcpy (out_address, ptr);

    /* next please */
    ptr = strtok (NULL, " \n");

  }

  /* remove angle brackets if the address has them */
  if (sscanf (out_address, "<%[^>]>", dump))
    strcpy (out_address, dump);

}

/*
 * Level 0 Routines
 */


/* open the mail folder */
result iomsg_open (void) {

  /* local variables */
  char folder_name[72]; /* name of folder file */
  char lock_name[72]; /* name of lock file */
  FILE *lock_file; /* lock file handle */

  /* get name of folder and lock files */
  config_getfolder (folder_name);
  config_getlockfile (lock_name);

  /* make sure the lock file isn't already there */
  if ((lock_file = fopen (lock_name, "r"))) {
    fclose (lock_file);
    return R_FILE;
  }

  /* not drop the lock file */
  if (!(lock_file = fopen (lock_name, "w"))) {
    return R_FILE;
  }
  fclose (lock_file);

  /* ensure the mailbox file is there, even if it's empty */
  if (!(folder_file = fopen (folder_name, "a"))) {
    return R_FILE;
  }
  fclose (folder_file);

  /* now open the folder file for reading */
  if (!(folder_file = fopen (folder_name, "r"))) {
    return R_FILE;
  }

  /* everything went OK so return */
  return R_OK;

}

/* close the mail folder */
result iomsg_close (void) {

  /* local variables */
  char folder_name[72]; /* name of folder file */
  char lock_name[72]; /* name of lock file */

  /* get name of folder and lock files */
  config_getfolder (folder_name);
  config_getlockfile (lock_name);

  /* close and delete */
  fclose (folder_file);
  remove (folder_name);
  remove (lock_name);

  /* all OK, return */
  return R_OK;

}

/* read a message */
result iomsg_read_header (void) {

  /* local variables */
  int reading_header = !0; /* !0 if currently reading header */
  char buffer[128]; /* buffer for reading message */
  char *ptr; /* pointer to look for line feeds */

  /* read header fields */
  while (reading_header && fgets (buffer, 128, folder_file)) {
    while ((ptr = strchr (buffer, '\n'))) *ptr = 0;
    if (*buffer == 0)
      reading_header = 0;
    else if (!(strncasecmp (buffer, "From: ", 6)))
      interpret_email_address (from_address, &buffer[6]);
    else if (!(strncasecmp (buffer, "Subject: ", 9))) {
      strncpy (game_name, &buffer[9], 15);
      game_name[15] = 0;
    }
  }

  /* return */
  if (reading_header) /* EOF met within header, oops */
    return R_EOF;
  else
    return R_OK;

}

/* return the address of the user */
char *iomsg_get_address (char *address) {
  return strcpy (address, from_address); 
}

/* return the game that they want to play */
char *iomsg_get_game(char *game) {
  return strcpy (game, game_name); 
}

/* read a line of the message body */
result iomsg_read_body(char *buffer) {

  /* local variables */
  char *ptr; /* pointer for looking for line feeds */
  static int last_line_contains_text; /* 0 if last line blank */

  /* read a line */
  if (!fgets (buffer, 128, folder_file))
    return R_EOF;
  else if (!strncmp (buffer, "From ", 5) && !last_line_contains_text)
    return R_EOF;
  else if (*buffer == '\n')
    last_line_contains_text = 0;
  else
    last_line_contains_text = !0;

  /* strip the line feed */
  if ((ptr = strchr (buffer, '\n'))) *ptr = 0;

  /* return OK */
  return R_OK;

}

/* post an outgoing message */
result iomsg_post(char *address, char *subject, char *text) {

  /* local variables */
  char mail_program[72]; /* mail program as specified in config */
  char command_line[128]; /* command line to feed pipe to */
  FILE *mail_input; /* handle for pipe to mail program */

  /* build up the command line */
  config_getmailprog (mail_program);
  if (strstr (mail_program, "%s"))
    sprintf (command_line, mail_program, subject);
  else
    strcpy (command_line, mail_program);
  strcat (command_line, " ");
  strcat (command_line, address);

  /* run the pipe */
  if (!(mail_input = popen (command_line, "w")))
    return R_FILE;
  fputs (text, mail_input);
  pclose (mail_input);

  /* everything went fine, yippee */
  return R_OK;

}
