/*
 * turnin.c
 * Copyright (C) Damian Walker 2000
 *
 * Astroserv - Inbound Order Processing
 *
 * $Id: turnin.c,v 1.3 2001/02/15 16:38:29 damian Exp $
 *
 * $Log: turnin.c,v $
 * Revision 1.3  2001/02/15 16:38:29  damian
 * Cleaned up logging and screen output
 *
 * Revision 1.2  2000/07/27 09:31:46  damian
 * Cosmetic changes to heading comments
 *
 * Revision 1.1.1.1  2000/07/27 09:18:13  damian
 * Initial release of Astroserv
 *
 * Revision 1.1  2000/05/31 10:17:52  damian
 * Initial revision
 *
 */


/* included headers ***************************************************/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <astrowar.h>
#include <astrowar/detail.h>
#include <astrowar/gamlst.h>
#include <astrowar/orders.h>
#include <astrowar/transn.h>
#include "astrosrv.h"  /* all sorts of defs for Astroserv */
#include "iomsg.h"    /* for input/output messages */
#include "user.h"     /* user file processing defs */
#include "config.h"   /* configuration file processing defs */
#include "plytxt.h"   /* temporary text file processing defs */
#include "pfleet.h"   /* player fleet processing defs */
#include "joinrq.h"   /* join request processing defs */
#include "player.h"   /* player processing defs */
#include "scrlog.h"   /* for screen and log file output */


/* enums, structures &c. **********************************************/


/* bit defs for what files need packing */
#define P_PLAYER 0x01
#define P_PFLEET 0x04


/* global variables ***************************************************/


int turnin_pack; /* which database files need packing? */


/* level 4 routines ***************************************************/


/* turnin_sendreply() - send reply to user */
void turnin_sendreply(user *u)
{
    char to_address[72], /* user to post to */
         subject[72], /* the subject */
         gamename[16], /* name of the game */
        *text; /* text message to send */

    /* return if no text to send */
    if( (text = plytxt_read( user_getnumber(u) )) == NULL ) return;

    /* set some message header fields */
    user_getaddr(to_address, u);
    gamlst_getname(gamename);
    sprintf(subject, "Reply to orders for game \"%s\"", gamename);

    /* post message */
    iomsg_post(to_address, subject, text);
    scrlog_replyposted(to_address);
}

/* turnin_interpretorder() - interpret user order */
char turnin_interpretorder(char *orderline)
{
    char logentry[1024];
    sprintf (logentry, "Processing Order: %s", orderline);
    scrlog_debug (logentry);
    if(*orderline == '\0' || *orderline == ';' || *orderline == '#')
                                                   return ';';
    if( !strncmp(orderline, "-- ", 3) ||
        !strcasecmp(orderline,  "-- ") )              return '-';
    if( !strncasecmp(orderline, "join", 4) )          return 'J';
    if( !strncasecmp(orderline, "send", 4) )          return 'S';
    if( !strncasecmp(orderline, "scout", 5) )         return 'C';
    if( !strncasecmp(orderline, "list planets", 12) ||
        !strncasecmp(orderline, "planets", 7) )       return 'P';
    if( !strncasecmp(orderline, "list fleets", 11) ||
        !strncasecmp(orderline, "fleets", 6) )        return 'F';
    if( !strncasecmp(orderline, "map", 3) )           return 'M';
    if( !strncasecmp(orderline, "write", 5) )         return 'W';
    if( !strncasecmp(orderline, "info", 4) )          return 'I';
    if( !strncasecmp(orderline, "resign", 6) )        return 'Q';
                                                   return '!';
}

/* turnin_rejectjoin() - reject join order from existing player */
void turnin_rejectjoin(user *u, char *orderline, int line)
{
    long number; /* user number */

    scrlog_debug ("Rejection join");
    number = user_getnumber(u);
    scrlog_badorder(line);
    plytxt_write(number, " > ");
    plytxt_write(number, orderline);
    plytxt_write(number, "\n\nYou are already playing this game\n\n");
}

/* turnin_processsend() - process a send order */
void turnin_processsend(user *u, char *orderline)
{
    order  *o = orders_new();           /* 'send' order */
    player *p = player_new();           /* player issuing send order */
    static
    char    word[7][72];                /* words in list */
    char    txtnum[11],                 /* player number as text */
            ename[16];                  /* empire name */
    long    number = user_getnumber(u), /* user number */
            ships;                      /* ships to send */
    int     wordctr;                    /* word counter */

    /* log and initialise */
    scrlog_processingorder("send");
    orders_settype(o, 'S');
    if(sscanf( orderline, "%s%s%s%s%s%s%s", word[0], word[1], word[2],
        word[3], word[4], word[5], word[6] ) < 4)
    {
        plytxt_write(number, " > ");
        plytxt_write(number, orderline);
        plytxt_write(number, "\n\nError in send.\n\n");
        orders_old(o);
	scrlog_debug ("Error in send (not enough parameters)");
        return;
    }
    word[1][15] = word[2][15] = word[3][15] = word[4][15] = word[5][15] =
    word[6][15] = '\0';

    /* set empire */
    sprintf(txtnum, "%ld", number);
    player_find(p, txtnum);
    player_getempire(ename, p);
    orders_setempire(o, ename);

    /* check number of ships */
    if(sscanf( word[1], "%ld", &ships ) != 1)
    {
        plytxt_write(number, " > ");
        plytxt_write(number, orderline);
        plytxt_write(number, "\n\nError in send.\n\n");
        player_old(p);
        orders_old(o);
	scrlog_debug ("Error in send (no quantity");
        return;
    }
    orders_setships(o, ships);
    if(!strcasecmp(word[2], "ships") || !strcasecmp(word[2], "ship"))
        wordctr = 3;
    else
        wordctr = 2;

    /* check planets of origin and destination */
    if(!strcasecmp( word[wordctr], "from" )) wordctr++;
    orders_setorigin(o, word[wordctr++]);
    if(!strcasecmp( word[wordctr], "to" )) wordctr++;
    orders_setdest(o, word[wordctr]);

    /* clean up and return */
    orders_write(o);
    orders_old(o);
    player_old(p);
    scrlog_debug ("Done processing send");
}

/* turnin_processscout() - process a scout order */
void turnin_processscout(user *u, char *orderline)
{
    order  *o = orders_new();           /* 'send' order */
    player *p = player_new();           /* player issuing send order */
    static
    char    word[7][72];                /* words in list */
    char    txtnum[11],                 /* player number as text */
            ename[16];                  /* empire name */
    long    number = user_getnumber(u); /* user number */

    /* log and initialise */
    scrlog_processingorder("scout");
    orders_settype(o, 'S');
    if(sscanf( orderline, "%s%s%s%s", word[0], word[1], word[2],
        word[3] ) < 3)
    {
        plytxt_write(number, " > ");
        plytxt_write(number, orderline);
        plytxt_write(number, "\n\nError in scout order.\n\n");
        orders_old(o);
        player_old(p);
	scrlog_debug ("Error in scout order (not enough parameters)");
        return;
    }
    word[1][15] = word[2][15] = word[3][15] = '\0';

    /* set empire and ships */
    sprintf(txtnum, "%ld", number);
    player_find(p, txtnum);
    player_getempire(ename, p);
    orders_setempire(o, ename);
    orders_setships(o, 1);

    /* check planets of origin and destination */
    orders_setdest( o, word[1] );
    if(strcasecmp( word[2], "from" ))
        orders_setorigin( o, word[2] );
    else
        orders_setorigin( o, word[3] );

    /* clean up and return */
    orders_write(o);
    orders_old(o);
    player_old(p);
    scrlog_debug ("Done processing scout");
}

/* turnin_processplanets() - process a planets order */
void turnin_processplanets(user *u)
{
    order  *o = orders_new();           /* 'send' order */
    player *p = player_new();           /* player issuing send order */
    char    txtnum[11],                 /* player number as text */
            ename[16];                  /* empire name */
    long    number = user_getnumber(u); /* user number */

    /* log and initialise */
    scrlog_processingorder("planets");
    orders_settype(o, 'P');

    /* set empire */
    sprintf(txtnum, "%ld", number);
    player_find(p, txtnum);
    player_getempire(ename, p);
    orders_setempire(o, ename);

    /* clean up and return */
    orders_write(o);
    orders_old(o);
    player_old(p);
    scrlog_debug ("Done processing planets");
}

/* turnin_processfleets() - process a fleets order */
void turnin_processfleets(user *u)
{
    order  *o = orders_new();           /* 'send' order */
    player *p = player_new();           /* player issuing send order */
    char    txtnum[11],                 /* player number as text */
            ename[16];                  /* empire name */
    long    number = user_getnumber(u); /* user number */

    /* log and initialise */
    scrlog_processingorder("fleets");
    orders_settype(o, 'F');

    /* set empire */
    sprintf(txtnum, "%ld", number);
    player_find(p, txtnum);
    player_getempire(ename, p);
    orders_setempire(o, ename);

    /* clean up and return */
    orders_write(o);
    orders_old(o);
    player_old(p);
    scrlog_debug ("Done processing fleets");
}

/* turnin_processmap() - process a map order */
void turnin_processmap(user *u, char *orderline)
{
    order  *o = orders_new();           /* 'send' order */
    player *p = player_new();           /* player issuing send order */
    static
    char    word[7][72];                /* words in list */
    char    txtnum[11],                 /* player number as text */
            ename[16];                  /* empire name */
    long    number = user_getnumber(u); /* user number */
    int     wordctr;                    /* word counter */

    /* log and initialise */
    scrlog_processingorder("map");
    orders_settype(o, 'M');
    if(sscanf( orderline, "%s%s%s%s", word[0], word[1], word[2],
        word[3] ) < 2)
    {
        plytxt_write(number, " > ");
        plytxt_write(number, orderline);
        plytxt_write(number, "\n\nError in map order.\n\n");
        orders_old(o);
        player_old(p);
	scrlog_debug ("Error in map order (not enough parameters)");
        return;
    }
    word[1][15] = word[2][15] = word[3][15] = '\0';

    /* set empire */
    sprintf(txtnum, "%ld", number);
    player_find(p, txtnum);
    player_getempire(ename, p);
    orders_setempire(o, ename);

    /* set centre planet */
    wordctr = 1;
    if(!strcasecmp( word[wordctr], "planets" )) wordctr++;
    if(!strcasecmp( word[wordctr], "from" ))    wordctr++;
    orders_setcentre( o, word[wordctr] );

    /* clean up and return */
    orders_write(o);
    orders_old(o);
    player_old(p);
    scrlog_debug ("Done processing map");
}

/* turnin_processwrite() - process a write order */
void turnin_processwrite(user *u, char *orderline)
{
    order  *o = orders_new();           /* 'write' order */
    player *p = player_new();           /* player writing message */
    char    ename[16],                  /* empire writing message */
            txtnum[11],                 /* user number as text */
            recip[72],                  /* recipient as requested */
           *text = malloc(1),           /* message text */
            msgline[128];               /* pointer to a message line */
    long    number = user_getnumber(u); /* user number */

    /* log and initialise */
    scrlog_processingorder("write");
    if(!strcasecmp(orderline, "write") || !strcasecmp(orderline, "write to"))
    {
        plytxt_write(number, " > ");
        plytxt_write(number, orderline);
        plytxt_write(number, "\n\nError in write order.\n\n");
        orders_old(o);
        player_old(p);
	scrlog_debug ("Error in write order");
        return;
    }
    *text = '\0';
    orders_settype(o, 'W');

    /* set empire */
    sprintf(txtnum, "%ld", number);
    player_find(p, txtnum);
    player_getempire(ename, p);
    orders_setempire(o, ename);

    /* set recipient */
    if(!strncasecmp( &orderline[6], "to ", 3 ))
        strcpy( recip, &orderline[9] );
    else
        strcpy( recip, &orderline[6] );
    if( !strcasecmp(recip, "all") )
        orders_setrecip(o, "*");
    else if( !strcasecmp(recip, "system") )
        orders_setrecip(o, "");
    else
        orders_setrecip(o, recip);

    /* set message text */
    while (iomsg_read_body (msgline) == R_OK)
    {
        if(!*msgline) break;
        text = realloc(text, strlen(text) + strlen(msgline) + 2);
        strcat(text, msgline);
        strcat(text, "\n");
    }
    orders_settext(o, text);

    /* clean up */
    orders_write(o);
    free(text);
    player_old(p);
    scrlog_debug ("Done processing write");

}

/* turnin_processinfo() - process an info order */
void turnin_processinfo(user *u)
{
    detail *d;        /* detail record to extract info from */
    char    path[72], /* path to game */
            name[36], /* name of game */
            buf[72];  /* buffer for line output */
    long    number;   /* user number */

    /* intialise */
    scrlog_processingorder("info");
    number = user_getnumber(u);

    /* attempt to read detail file */
    gamlst_getpath(path);
    if( detail_open(&d, path) ) return;

    /* write detail entries to user */
    detail_getname(name, d);
    sprintf(buf, "Settings for game \"%s\"\n", name);
    plytxt_write(number, buf);
    sprintf( buf, "Planets:             %ld\n", detail_getplanets(d) );
    plytxt_write(number, buf);
    sprintf( buf, "Galaxy size:         %d,%d..%d,%d\n",
        detail_getxmin(d), detail_getymin(d), detail_getxmax(d),
        detail_getymax(d));
    plytxt_write(number, buf);
    sprintf( buf, "Max travel distance: %d\n", detail_getmaxdist(d) );
    plytxt_write(number, buf);
    sprintf( buf, "Ship speed:          %d\n\n", detail_getspeed(d) );
    plytxt_write(number, buf);

    /* close detail file */
    detail_close(d);
    scrlog_debug ("Done processing info");
}

/* turnin_processresign() - process a 'resign' order */
void turnin_processresign(user *u)
{
    order  *o = orders_new();           /* order record */
    player *p = player_new();           /* player issuing order */
    pfleet *f = pfleet_new();           /* player fleet record */
    result  r;                          /* result of fleet read */
    char    txtnum[11],                 /* user number as text */
            ename[16];                  /* name of empire */
    long    number = user_getnumber(u); /* user number */

    /* log and initialise */
    scrlog_processingorder("resign");
    orders_settype(o, 'R');

    /* set empire */
    sprintf(txtnum, "%ld", number);
    player_find(p, txtnum);
    player_getempire(ename, p);
    orders_setempire(o, ename);
    orders_write(o);

    /* delete player record */
    player_delete(p);
    turnin_pack |= P_PLAYER;

    /* delete player fleets */
    r = pfleet_first(f, F_FLEETNO);
    while(!r)
    {
        if( pfleet_getuser(f) == number ) pfleet_delete(f);
        r = pfleet_next(f, F_FLEETNO);
    }
    turnin_pack |= P_PFLEET;

    /* send notification */
    plytxt_write(number, " > resign\n\n"
                         "You have been removed from this game.\n\n");

    /* return */
    orders_old(o);
    pfleet_old(f);
    player_old(p);
    scrlog_debug ("Done processing resign");
}

/* turnin_processjoin() - process a join order */
void turnin_processjoin(user *u, char *orderline)
{
    char   *ptr,              /* current position in line */
            path[72];         /* path to game files */
    order  *o = orders_new(); /* order to write */
    long    trans,            /* transaction number */
            number;           /* user number */
    joinrq *j = joinrq_new(); /* join request record */
    result  r;                /* for scanning */

    /* check there are parameters */
    number = user_getnumber(u);
    if( !strcasecmp(orderline, "join") )
    {
        plytxt_write(number, " > join\n\n"
                             "You need to specify an empire name\n\n");
    }

    /* log & initialise */
    scrlog_processingorder("join");
    gamlst_getpath(path);

    /* check user is not already in join request file */
    r = joinrq_first(j, J_NONE);
    while(r == R_OK)
    {
        if( joinrq_getuser(j) == user_getnumber(u) )
        {
            plytxt_write(number, " > ");
            plytxt_write(number, orderline);
            plytxt_write(number, "\n\nYou have already reqested to "
                                 "join this game\n\n");
            orders_old(o);
            joinrq_old(j);
	    scrlog_debug ("Error in join (already in queue)");
            return;
        }
        r = joinrq_next(j, J_NONE);
    }

    /* parse line */
    ptr = &orderline[5]; /* skip 'join' */
    if( !strncasecmp(ptr, "as ", 3) ) ptr = &ptr[3];

    /* set order fields */
    if( strlen(ptr) > 15 ) ptr[15] = '\0';
    orders_settype(  o, 'J');
    orders_settrans( o, trans = transn_read(path) );
    orders_setempire(o, ptr);
    orders_write(    o);

    /* log the join request */
    joinrq_clear(    j);
    joinrq_settrans( j, trans);
    joinrq_setuser(  j, number);
    joinrq_setempire(j, ptr);
    joinrq_write(    j);

    /* clean up */
    joinrq_old(j);
    orders_old(o);
    scrlog_debug ("Done processing join");
}

/* turnin_rejectorder() - reject an order */
void turnin_rejectorder(user *u, char *orderline, int line)
{
    long number; /* user number */

    number = user_getnumber(u);
    scrlog_badorder(line);
    plytxt_write(number, " > ");
    plytxt_write(number, orderline);
    plytxt_write(number, "\n\n");
    plytxt_write(number, "Order not valid until player accepted\n\n");
    scrlog_debug ("Order not valid");
}

/* turnin_error() - command error in input message */
void turnin_error(user *u, char *orderline, int line)
{
    long number; /* user number */

    number = user_getnumber(u);
    scrlog_badorder(line);
    plytxt_write(number, " > ");
    plytxt_write(number, orderline);
    plytxt_write(number, "\n\nCannot understand this order\n\n");
    scrlog_debug ("Nonsense order");
}


/* level 3 routines ***************************************************/


/* turnin_opengame() - open game files */
result turnin_opengame(void)
{
    result r;        /* result of open attempt */
    char   path[72]; /* path to game files */

    /* establish path */
    gamlst_getpath(path);

    /* attempt to open player file */
    switch( r = player_open(path) )
    {
        case R_FILE:   if(( r = player_create(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "splayer.data");
                           return R_FILE;
                       }
                       scrlog_filecreated(path, "splayer.data");
                       if(( r = player_open(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "splayer.data");
                           return R_FILE;
                       }
                       scrlog_fileopened(path, "splayer.data");
                       break;
        case R_HEADER: scrlog_badheaderinfile(path, "splayer.data");
                       return R_HEADER;
        case R_OK:     scrlog_fileopened(path, "splayer.data");
                       break;
        default:       scrlog_badreturncode(r, "player_open");
                       return R_BADCMD;
    }

    /* attempt to open player fleet file */
    switch( r = pfleet_open(path) )
    {
        case R_FILE:   if(( r = pfleet_create(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "spfleet.data");
                           goto closeplayer;
                       }
                       scrlog_filecreated(path, "spfleet.data");
                       if(( r = pfleet_open(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "spfleet.data");
                           goto closeplayer;
                       }
                       scrlog_fileopened(path, "spfleet.data");
                       break;
        case R_HEADER: scrlog_badheaderinfile(path, "spfleet.data");
                       goto closeplayer;
        case R_OK:     scrlog_fileopened(path, "spfleet.data");
                       break;
        default:       scrlog_badreturncode(r, "pfleet_open");
                       goto closeplayer;
    }

    /* attempt to open join request file */
    switch( r = joinrq_open(path) )
    {
        case R_FILE:   if(( r = joinrq_create(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "sjoinrq.data");
                           goto closepfleet;
                       }
                       scrlog_filecreated(path, "sjoinrq.data");
                       if(( r = joinrq_open(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "sjoinrq.data");
                           goto closepfleet;
                       }
                       scrlog_fileopened(path, "sjoinrq.data");
                       break;
        case R_HEADER: scrlog_badheaderinfile(path, "sjoinrq.data");
                       goto closepfleet;
        case R_OK:     scrlog_fileopened(path, "sjoinrq.data");
                       break;
        default:       scrlog_badreturncode(r, "joinrq_open");
                       goto closepfleet;
    }

    /* attempt to open orders file */
    switch( r = orders_open(path) )
    {
        case R_FILE:  if(( r = orders_create(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "orders.data");
                           r = R_FILE;
                           goto closejoinrq;
                       }
                       scrlog_filecreated(path, "orders.data");
                       if(( r = orders_open(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "orders.data");
                           r = R_FILE;
                           goto closejoinrq;
                       }
                       scrlog_fileopened(path, "orders.data");
                       break;
        case R_HEADER: scrlog_badheaderinfile(path, "orders.data");
                       r = R_HEADER;
                       goto closejoinrq;
        case R_OK:     scrlog_fileopened(path, "orders.data");
                       break;
        default:       scrlog_badreturncode(r, "orders_open");
                       goto closejoinrq;
    }

    /* all OK so return */
    plytxt_open(path);
    turnin_pack = 0;
    return R_OK;

    /* clean up after fatal error */
    closejoinrq:
        joinrq_close();
    closepfleet:
        pfleet_close();
    closeplayer:
        player_close();
    return r;
}

/* process existing player's record */
void turnin_existingplayer (user *u) {

  /* local variables */
  char orderline[128]; /* line of order message */
  int line = 1, /* line number */
    eot = 0; /* end of text flag */
  result r; /* result of reading a line */

  /* process order lines */
  r = iomsg_read_body (orderline);
  while (r == R_OK && !eot) {
    switch (turnin_interpretorder (orderline)) {
    case 'J': turnin_rejectjoin (u, orderline, line); break;
    case 'S': turnin_processsend (u, orderline);      break;
    case 'C': turnin_processscout (u, orderline);     break;
    case 'P': turnin_processplanets (u);              break;
    case 'F': turnin_processfleets (u);               break;
    case 'M': turnin_processmap (u, orderline);       break;
    case 'W': turnin_processwrite (u, orderline);     break;
    case 'I': turnin_processinfo (u);                 break;
    case 'Q': turnin_processresign (u); eot = 1;      break;
    case ';': /* ignore comments etc. */              break;
    case '-': eot = 1;                                break;
    default:  turnin_error (u, orderline, line);      break;
    }
    line++;
    r = iomsg_read_body (orderline);
  }

  /* skip everything after the '-- ' line */
  if (eot)
    while (r == R_OK)
      r = iomsg_read_body (orderline);

  /* clean up and send reply */
  scrlog_messagedone();
  turnin_sendreply(u);

}

/* turnin_newplayer() - process new player's record */
void turnin_newplayer(user *u)
{

  /* local variables */
  char orderline[128]; /* line of order message */
  int line = 1, /* line number */
    eot = 0; /* end of text flag */
  result r; /* result of reading a line */

  /* process order lines */
  r = iomsg_read_body (orderline);
  while (r == R_OK && !eot) {
    switch (turnin_interpretorder (orderline)) {
    case 'J': turnin_processjoin (u, orderline);       break;
    case 'S':
    case 'C':
    case 'P':
    case 'F':
    case 'M':
    case 'Q':
    case 'W': turnin_rejectorder (u, orderline, line); break;
    case 'I': turnin_processinfo (u);                  break;
    case ';': /* ignore comments etc. */               break;
    case '-': eot = 1;                                 break;
    default:  turnin_error (u, orderline, line);       break;
    }
    line++;
    r = iomsg_read_body (orderline);
  }

  /* skip everything after the '-- ' line */
  if (eot)
    while (r == R_OK)
      r = iomsg_read_body (orderline);

  /* clean up and send reply */
  scrlog_messagedone();
  turnin_sendreply(u);

}

/* turnin_closegame() - close the game files */
void turnin_closegame(void)
{
    char path[72]; /* path to game files */

    /* close files */
    player_close();
    pfleet_close();
    joinrq_close();
    orders_close();
    plytxt_close();

    /* pack files if necessary */
    gamlst_getpath(path);
    if(turnin_pack & P_PLAYER) player_pack(path);
    if(turnin_pack & P_PFLEET) pfleet_pack(path);
}


/* level 2 routines ***************************************************/


/* ensure that user exists */
result turnin_checkuser (user *u, char *address) {

  /* local variables */
  user *last = user_new(); /* last user in file */

  /* add user if not found */
  if (user_find (u, address) != R_OK) {
    if (user_last (last, U_NUMBER) == R_OK)
      user_setnumber (u, user_getnumber (last) + 1);
    else
      user_setnumber (u, 1);
    user_setaddr (u, address);
    user_write (u);
  }

  /* clean up and return */
  user_old (last);
  return R_OK;
}

/* send user a 'game not found' message */
void turnin_gamenotfound (user *u, char *gamename) {

  /* local variables */
  char to_address[72], /* user to post to */
    buffer[128], /* buffer for inbound message body lines */
    text[1024], /* text message to send */
    readname[16], /* name of game read from file */
    outname[19]; /* name of game to output */
  int c; /* line counter for output */
  result r; /* result of game list read */

  /* set message header and heading text */
  iomsg_get_address (to_address);
  sprintf(text,
	  "Sorry, the game \"%s\" is not one of the games on offer at "
	  "this system.\nA full list of available games is shown below:"
	  "\n\n", gamename);

  /* build game list */
  c = 0;
  r = gamlst_first();
  while(r == R_OK && strlen(text) < 1000) {
    gamlst_getname(readname);
    sprintf(outname, "%18s", readname);
    strcat(text, outname);
    if(++c > 3) strcat(text, "\n");
    c %= 4;
    r = gamlst_next();
  }
  if(c) strcat(text, "\n");

  /* post message */
  iomsg_post (to_address, "Game not found", text);

  /* skip the body of the inbound message */
  while (iomsg_read_body (buffer) == R_OK);

}

/* turnin_processorderlist() - process an order list */
result turnin_processorderlist(user *u)
{
    player *p = player_new(); /* player record */
    char    searchtext[12];   /* for searching player file */
    result  r;                /* returned to calling process */

    /* open game files */
    if(( r = turnin_opengame() ) != R_OK )
    {
        player_old(p);
        return r;
    }

    /* process new or existing player */
    sprintf( searchtext, "%ld", user_getnumber(u) );
    if(player_find(p, searchtext) == R_OK)
        turnin_existingplayer(u);
    else
        turnin_newplayer(u);

    /* clean up and return */
    turnin_closegame();
    player_old(p);
    return R_OK;
}


/* level 1 routines ***************************************************/


/* turnin_interpret() - interpret command line arguments */
result turnin_interpret(char **argp) {
  if( argp[0] ) {
    scrlog_error();
    return R_BADCMD;
  }
  return R_OK;
}

/* turnin_open() - open the turnin process */
result turnin_open(char *path)
{
    result r;            /* result returned to calling process */
    int    line;         /* file line number for log */
    int    loglevel;     /* level of log I/O */
    char   folder[128],  /* netmail directory */
           logfile[128]; /* name of log file */

    /* attempt to open config */
    config_open(path);
    switch( r = config_read(path, &line) )
    {
        case R_FILE:   scrlog_cannotreadconfig(path);
                       return R_FILE;
        case R_BADCMD: scrlog_badcommandinconfig(path, line);
                       return R_BADCMD;
        case R_OK:     config_getlogfile(logfile);
		       loglevel = config_getloglevel ();
                       if( ( r = scrlog_openlog(logfile, loglevel) )
                           != R_OK)
                       {
                           scrlog_cannotopenlog(logfile);
                           goto closeconfig;
                       }
                       scrlog_turnin();
                       scrlog_configread(path);
                       break;
        default:       scrlog_badreturncode(r, "config_read");
                       return r;
    }

    /* attempt to open input message lib */
    switch( r = iomsg_open() )
    {
        case R_FILE:   config_getfolder(folder);
                       scrlog_cannotopenfolder(folder);
                       goto closeconfig;
        case R_MEMORY: scrlog_outofmemory("opening message folder");
                       goto closeconfig;
        case R_OK:     config_getfolder(folder);
                       scrlog_folderopened(folder);
                       break;
        default:       scrlog_badreturncode(r, "iomsg_open");
                       goto closeconfig;
    }

    /* attempt to open game list */
    switch( r = gamlst_open(path, &line) )
    {
        case R_FILE:   scrlog_cannotopengamelist(path);
                       goto closeiomsg;
        case R_EOF:    scrlog_badfileingamelist(path, line);
                       goto closeiomsg;
        case R_BADCMD: scrlog_badlineingamelist(path, line);
                       goto closeiomsg;
        case R_OK:     scrlog_gamelistread(path);
                       break;
        default:       scrlog_badreturncode(r, "gamlst_open");
                       goto closeiomsg;
    }

    /* attempt to open user file */
    switch( r = user_open(path) )
    {
        case R_FILE:   if(( r = user_create(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "suser.data");
                           r = R_FILE;
                           goto closegamelist;
                       }
                       scrlog_filecreated(path, "suser.data");
                       if(( r = user_open(path) ) != R_OK)
                       {
                           scrlog_cannotopenfile(path, "suser.data");
                           r = R_FILE;
                           goto closegamelist;
                       }
                       scrlog_fileopened(path, "suser.data");
                       break;
        case R_HEADER: scrlog_badheaderinfile(path, "suser.data");
                       goto closegamelist;
        case R_OK:     scrlog_fileopened(path, "suser.data");
                       break;
        default:       scrlog_badreturncode(r, "user_open");
                       goto closegamelist;
    }

    /* all OK so return */
    return R_OK;

    /* clean up after fatal error */
    closegamelist:
        gamlst_close();
    closeiomsg:
        iomsg_close();
    closeconfig:
        config_close();
    return r;
}

/* process a single message */
result turnin_processmsg (void) {

  /* local variables */
  user *u = user_new(); /* user record */
  char  gamename[72], /* game named on subject line */
    from_address[72]; /* user address for log & screen */

  /* extract information from message */
  iomsg_get_address (from_address);
  scrlog_msgread (from_address);
  iomsg_get_game (gamename);

  /* check user */
  if( turnin_checkuser(u, from_address) == R_MEMORY ) {
    scrlog_outofmemory("checking user");
    user_old(u);
    return R_MEMORY;
  }

  /* process message */
  if( gamlst_find(gamename) != R_OK )
    turnin_gamenotfound(u, gamename);
  else
    turnin_processorderlist(u);

  /* clean up & return */
  user_old(u);
  return R_OK;

}

/* turnin_close() - close program */
void turnin_close(void)
{
    config_close();
    iomsg_close();
    gamlst_close();
    user_close();
    scrlog_finished();
}


/* level 0 routines ***************************************************/


/* turnin_main() - main turn processing */
result turnin_main(char *path, char **argp)
{
    result r; /* returned to calling process */

    if(( r = turnin_interpret(argp) )) return r;
    if(( r = turnin_open(path) ) != R_OK) return r;
    scrlog_scanning(); r = iomsg_read_header();
    while(r == R_OK)
    {
        if(( r = turnin_processmsg() ) != R_OK)
        {
            turnin_close();
            return r;
        }
        scrlog_scanning(); r = iomsg_read_header();
    }
    turnin_close();
    return R_OK;
}
